/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <stdio.h>
#include <clack/c/clack_types.h>
#include <oskit/net/ether.h>
#include <clack/c/packet.h> /* XXX should be stats.h */
#ifdef MEASURE
#include <clack/c/stats.h>
#endif

/*
 * Imports!
 */
extern unsigned char ethaddr0[ETHER_ADDR_SIZE];
extern unsigned char ethaddr1[ETHER_ADDR_SIZE];
extern void		schedule();

int main(int argc, char* argv[])
{
        printf("Clack kernel\n");
        printf("eth0: "); print_ether(ethaddr0); printf("\n");
        printf("eth1: "); print_ether(ethaddr1); printf("\n");

#ifdef  PERFMON
#if 1
	enable_perfcounter(0, PMC6_INST_RETIRED,         0, PMCF_OS|PMCF_USR);
#define COUNTER0_STRING "Inst Retired: " 
	enable_perfcounter(1, PMC6_UOPS_RETIRED,         0, PMCF_OS|PMCF_USR);
#define COUNTER1_STRING "UOPs Retired: "
#endif
#if 0
	enable_perfcounter(0, PMC6_BTB_MISSES,           0, PMCF_OS|PMCF_USR);
#define COUNTER0_STRING "BTB Misses:  " 
	enable_perfcounter(1, PMC6_DCU_MISS_OUTSTANDING, 0, PMCF_OS|PMCF_USR);
#define COUNTER1_STRING "DCU Miss Outstanding: "
#endif
#if 0
	enable_perfcounter(0, PMC6_BR_INST_RETIRED,      0, PMCF_OS|PMCF_USR);
#define COUNTER0_STRING "Branches Retired:  " 
	enable_perfcounter(1, PMC6_BR_MISS_PRED_RETIRED, 0, PMCF_OS|PMCF_USR);
#define COUNTER1_STRING "Branches MissPred: "
#endif
#if 0
	enable_perfcounter(0, PMC6_IFU_IFETCH_MISS, 0, PMCF_OS|PMCF_USR);
#define COUNTER0_STRING "IFetch Misses:" 
	enable_perfcounter(1, PMC6_L2_IFETCH,     0xF, PMCF_OS|PMCF_USR);
#define COUNTER1_STRING "L2 Ifetches:  "
#endif
#if 0
	enable_perfcounter(0, PMC6_IFU_IFETCH_MISS, 0, PMCF_OS|PMCF_USR);
#define COUNTER0_STRING "IFetch Misses:" 
	enable_perfcounter(1, PMC6_L2_IFETCH,     0xF, PMCF_OS|PMCF_USR);
#define COUNTER1_STRING "L2 Ifetches:  "
#endif
#endif	
        while (1) {
		/*
		 * Packets come in!
		 */

		schedule();

#ifdef MEASURE
		if (packet_count > 10000) {
			printf("Stats: %d packets at %d cycles per packet\n",
			       packet_count, packet_total / packet_count);
#ifdef PERFMON
			printf("Perfmon: %s %d (%d per packet)\n", COUNTER0_STRING,
			       (int) perfmon_counter0, (int) (perfmon_counter0 / packet_count));
			printf("Perfmon: %s %d (%d per packet)\n", COUNTER1_STRING,
			       (int) perfmon_counter1, (int) (perfmon_counter1 / packet_count));
#endif
			return 0;
		}
#endif
        }
	return 0;
}
