/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/*
 * Packet Queues.
 */
#include <clack/c/packet.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

/*
 * Imported.
 */
extern int		qsize;
extern clack_error_t	outgoing_push(clack_packet_t *);

/*
 * Exported.
 */
clack_error_t		incoming_push(clack_packet_t *);
clack_error_t		schedule();

/*
 * Our queue is a simple circular buffer!
 */
static clack_packet_t	**myqueue;
static int		head, tail;

/*
 * Allocate our queue.
 */
clack_error_t
init(void)
{
	myqueue = (clack_packet_t **) malloc(sizeof(clack_packet_t *) * qsize);
	assert(myqueue);

	return 0;
}

/*
 * A new packet is placed in the queue.
 */
clack_error_t
incoming_push(clack_packet_t *p)
{
	int		next = ((tail != qsize) ? tail + 1 : 0);

	if (head != next) {
		myqueue[tail] = p;
		tail = next;
	}
	else
		clack_packet_release(p);

	return 0;
}

/*
 * Try to send off a packet downstream.
 */
clack_error_t
schedule(void)
{
	int		next;
	clack_packet_t *p;

	if (head != tail) {
		next = ((head != qsize) ? head + 1 : 0);
		
		p = myqueue[head];
		assert(p);
		head = next;
		return outgoing_push(p);
	}
	return 0;
}
